@@ -103,7 +103,7 @@ int nfct_setobjopt(struct nf_conntrack *ct, unsigned int option)
 {
 	assert(ct != NULL);
 
-	if (option > NFCT_SOPT_MAX) {
+	if (unlikely(option > NFCT_SOPT_MAX)) {
 		errno = EOPNOTSUPP;
 		return -1;
 	}
@@ -123,7 +123,7 @@ int nfct_getobjopt(const struct nf_conntrack *ct, unsigned int option)
 {
 	assert(ct != NULL);
 
-	if (option > NFCT_GOPT_MAX) {
+	if (unlikely(option > NFCT_GOPT_MAX)) {
 		errno = EOPNOTSUPP;
 		return -1;
 	}
@@ -206,9 +206,9 @@ void nfct_callback_unregister(struct nfct_handle *h)
  * @value: pointer to the attribute value
  *
  * Note that certain attributes are unsettable:
- * 	- ATTR_USE
- * 	- ATTR_ID
- * 	- ATTR_*_COUNTER_*
+ * 	ATTR_USE
+ * 	ATTR_ID
+ * 	ATTR_*_COUNTER_*
  * The call of this function for such attributes do nothing.
  */
 void nfct_set_attr(struct nf_conntrack *ct,
@@ -218,7 +218,7 @@ void nfct_set_attr(struct nf_conntrack *ct,
 	assert(ct != NULL);
 	assert(value != NULL);
 
-	if (type >= ATTR_MAX)
+	if (unlikely(type >= ATTR_MAX))
 		return;
 
 	if (set_attr_array[type]) {
@@ -279,7 +279,7 @@ const void *nfct_get_attr(const struct nf_conntrack *ct,
 {
 	assert(ct != NULL);
 
-	if (type >= ATTR_MAX) {
+	if (unlikely(type >= ATTR_MAX)) {
 		errno = EINVAL;
 		return NULL;
 	}
@@ -355,13 +355,41 @@ int nfct_attr_is_set(const struct nf_conntrack *ct,
 {
 	assert(ct != NULL);
 
-	if (type >= ATTR_MAX) {
+	if (unlikely(type >= ATTR_MAX)) {
 		errno = EINVAL;
 		return -1;
 	}
 	return test_bit(type, ct->set);
 }
 
+/**
+ * nfct_attr_is_set_array - check if an array of attribute types is set
+ * @ct: pointer to a valid conntrack object
+ * @array: attribute type array
+ * @size: size of the array
+ *
+ * On error, -1 is returned and errno is set appropiately, otherwise
+ * the value of the attribute is returned.
+ */
+int nfct_attr_is_set_array(const struct nf_conntrack *ct,
+			   const enum nf_conntrack_attr *type_array,
+			   int size)
+{
+	int i;
+
+	assert(ct != NULL);
+
+	for (i=0; i<size; i++) {
+		if (unlikely(type_array[i] >= ATTR_MAX)) {
+			errno = EINVAL;
+			return -1;
+		}
+		if (!test_bit(type_array[i], ct->set))
+			return 0;
+	}
+	return 1;
+}
+
 /**
  * nfct_attr_unset - unset a certain attribute
  * @type: attribute type
@@ -375,7 +403,7 @@ int nfct_attr_unset(struct nf_conntrack *ct,
 {
 	assert(ct != NULL);
 
-	if (type >= ATTR_MAX) {
+	if (unlikely(type >= ATTR_MAX)) {
 		errno = EINVAL;
 		return -1;
 	}
@@ -384,6 +412,99 @@ int nfct_attr_unset(struct nf_conntrack *ct,
 	return 0;
 }
 
+/**
+ * nfct_set_attr_grp - set a group of attributes
+ * @ct: pointer to a valid conntrack object
+ * @type: attribute group (see ATTR_GRP_*)
+ * @data: pointer to struct (see struct nfct_attr_grp_*)
+ *
+ * Note that calling this function for ATTR_GRP_COUNTER_* does nothing since 
+ * counters are unsettable.
+ */
+void nfct_set_attr_grp(struct nf_conntrack *ct,
+		       const enum nf_conntrack_attr_grp type,
+		       const void *data)
+{
+	assert(ct != NULL);
+
+	if (unlikely(type >= ATTR_GRP_MAX))
+		return;
+
+	if (set_attr_grp_array[type]) {
+		set_attr_grp_array[type](ct, data);
+		set_bitmask_u32(ct->set, attr_grp_bitmask[type], __NFCT_BITSET);
+	}
+}
+
+/**
+ * nfct_get_attr_grp - get an attribute group
+ * @ct: pointer to a valid conntrack object
+ * @type: attribute group (see ATTR_GRP_*)
+ * @data: pointer to struct (see struct nfct_attr_grp_*)
+ *
+ * On error, it returns -1 and errno is appropriately set. On success, the
+ * data pointer contains the attribute group.
+ */
+int nfct_get_attr_grp(const struct nf_conntrack *ct,
+		      const enum nf_conntrack_attr_grp type,
+		      void *data)
+{
+	assert(ct != NULL);
+
+	if (unlikely(type >= ATTR_GRP_MAX)) {
+		errno = EINVAL;
+		return -1;
+	}
+	if (!test_bitmask_u32(ct->set, attr_grp_bitmask[type], __NFCT_BITSET)) {
+		errno = ENODATA;
+		return -1;
+	}
+	assert(get_attr_grp_array[type]);
+	get_attr_grp_array[type](ct, data);
+	return 0;
+}
+
+/**
+ * nfct_attr_grp_is_set - check if an attribute group is set
+ * @ct: pointer to a valid conntrack object
+ * @type: attribute group (see ATTR_GRP_*)
+ *
+ * If the attribute group is set, this function returns 1, otherwise 0.
+ */
+int nfct_attr_grp_is_set(const struct nf_conntrack *ct,
+			 const enum nf_conntrack_attr_grp type)
+{
+	assert(ct != NULL);
+
+	if (unlikely(type >= ATTR_GRP_MAX)) {
+		errno = EINVAL;
+		return -1;
+	}
+	return test_bitmask_u32(ct->set, attr_grp_bitmask[type], __NFCT_BITSET);
+}
+
+/**
+ * nfct_attr_grp_unset - unset an attribute group
+ * @ct: pointer to a valid conntrack object
+ * @type: attribute group (see ATTR_GRP_*)
+ *
+ * On error, it returns -1 and errno is appropriately set. On success, 
+ * this function returns 0.
+ */
+int nfct_attr_grp_unset(struct nf_conntrack *ct,
+			const enum nf_conntrack_attr_grp type)
+{
+	assert(ct != NULL);
+
+	if (unlikely(type >= ATTR_GRP_MAX)) {
+		errno = EINVAL;
+		return -1;
+	}
+	unset_bitmask_u32(ct->set, attr_grp_bitmask[type], __NFCT_BITSET);
+
+	return 0;
+}
+
 /**
  * nfct_build_conntrack - build a netlink message from a conntrack object
  * @ssh: nfnetlink subsystem handler
@@ -471,7 +592,7 @@ int nfct_build_query(struct nfnl_subsys_handle *ssh,
 		nfct_build_conntrack(ssh, req, size, IPCTNL_MSG_CT_NEW, NLM_F_REQUEST|NLM_F_ACK, data);
 		break;
 	case NFCT_Q_DESTROY:
-		nfct_build_conntrack(ssh, req, size, IPCTNL_MSG_CT_DELETE, NLM_F_ROOT|NLM_F_MATCH|NLM_F_REQUEST|NLM_F_ACK, data);
+		nfct_build_conntrack(ssh, req, size, IPCTNL_MSG_CT_DELETE, NLM_F_REQUEST|NLM_F_ACK, data);
 		break;
 	case NFCT_Q_GET:
 		nfct_build_conntrack(ssh, req, size, IPCTNL_MSG_CT_GET, NLM_F_REQUEST|NLM_F_ACK, data);
@@ -480,10 +601,10 @@ int nfct_build_query(struct nfnl_subsys_handle *ssh,
 		nfnl_fill_hdr(ssh, &req->nlh, 0, *family, 0, IPCTNL_MSG_CT_DELETE, NLM_F_REQUEST|NLM_F_ACK);
 		break;
 	case NFCT_Q_DUMP:
-		nfnl_fill_hdr(ssh, &req->nlh, 0, *family, 0, IPCTNL_MSG_CT_GET, NLM_F_ROOT|NLM_F_MATCH|NLM_F_REQUEST|NLM_F_DUMP);
+		nfnl_fill_hdr(ssh, &req->nlh, 0, *family, 0, IPCTNL_MSG_CT_GET, NLM_F_REQUEST|NLM_F_DUMP);
 		break;
 	case NFCT_Q_DUMP_RESET:
-		nfnl_fill_hdr(ssh, &req->nlh, 0, *family, 0, IPCTNL_MSG_CT_GET_CTRZERO, NLM_F_ROOT|NLM_F_MATCH|NLM_F_REQUEST|NLM_F_DUMP);
+		nfnl_fill_hdr(ssh, &req->nlh, 0, *family, 0, IPCTNL_MSG_CT_GET_CTRZERO, NLM_F_REQUEST|NLM_F_DUMP);
 		break;
 	case NFCT_Q_CREATE_UPDATE:
 		nfct_build_conntrack(ssh, req, size, IPCTNL_MSG_CT_NEW, NLM_F_REQUEST|NLM_F_CREATE|NLM_F_ACK, data);
@@ -640,13 +761,13 @@ int nfct_catch(struct nfct_handle *h)
  * If you pass NFCT_T_UNKNOWN, the message type will not be output. 
  *
  * Currently, the output available are:
- * 	- NFCT_O_DEFAULT: default /proc-like output
- * 	- NFCT_O_XML: XML output
+ * 	NFCT_O_DEFAULT: default /proc-like output
+ * 	NFCT_O_XML: XML output
  *
  * The output flags are:
- * 	- NFCT_OF_SHOW_LAYER3: include layer 3 information in the output, 
- * 	   this is *only* required by NFCT_O_DEFAULT.
- * 	- NFCT_OF_TIME: display time.
+ * 	NFCT_OF_SHOW_LAYER3: include layer 3 information in the output, 
+ * 	this is *only* required by NFCT_O_DEFAULT.
+ * 	NFCT_OF_TIME: display time.
  *
  * This function returns the size of the information that _would_ have been 
  * written to the buffer, even if there was no room for it. Thus, the
@@ -703,26 +824,25 @@ int nfct_compare(const struct nf_conntrack *ct1,
  *
  * The available flags are:
  *
- * 	- NFCT_CMP_STRICT: the compared objects must have the same attributes
+ * 	NFCT_CMP_STRICT: the compared objects must have the same attributes
  * 	and the same values, otherwise it returns that the objects are 
  * 	different.
- * 	- NFCT_CMP_MASK: the first object is used as mask, this means that 
+ * 	NFCT_CMP_MASK: the first object is used as mask, this means that 
  * 	if an attribute is present in ct1 but not in ct2, this function 
  * 	returns that the objects are different.
- * 	- NFCT_CMP_ALL: full comparison of both objects
- * 	- NFCT_CMP_ORIG: it only compares the source and destination address;
+ * 	NFCT_CMP_ALL: full comparison of both objects
+ * 	NFCT_CMP_ORIG: it only compares the source and destination address;
  * 	source and destination ports; the layer 3 and 4 protocol numbers
  * 	of the original direction; and the id (if present).
- * 	- NFCT_CMP_REPL: like NFCT_CMP_REPL but it compares the flow
+ * 	NFCT_CMP_REPL: like NFCT_CMP_REPL but it compares the flow
  * 	information that goes in the reply direction.
- * 	- NFCT_CMP_TIMEOUT_EQ: timeout(ct1) == timeout(ct2)
- * 	- NFCT_CMP_TIMEOUT_GT: timeout(ct1) > timeout(ct2)
- * 	- NFCT_CMP_TIMEOUT_LT: timeout(ct1) < timeout(ct2)
- * 	- NFCT_CMP_TIMEOUT_GE: timeout(ct1) >= timeout(ct2)
- * 	- NFCT_CMP_TIMEOUT_LE: timeout(ct1) <= timeout(ct2)
+ * 	NFCT_CMP_TIMEOUT_EQ: timeout(ct1) == timeout(ct2)
+ * 	NFCT_CMP_TIMEOUT_GT: timeout(ct1) > timeout(ct2)
+ * 	NFCT_CMP_TIMEOUT_LT: timeout(ct1) < timeout(ct2)
+ * 	NFCT_CMP_TIMEOUT_GE: timeout(ct1) >= timeout(ct2)
+ * 	NFCT_CMP_TIMEOUT_LE: timeout(ct1) <= timeout(ct2)
  *
- * The default status bits comparison consists of the following operation:
- * status(ct1) & status(ct2) == status(ct1).
+ * The status bits comparison is status(ct1) & status(ct2) == status(ct1).
  *
  * If both conntrack object are equal, this function returns 1, otherwise
  * 0 is returned.
@@ -750,14 +870,14 @@ int nfct_cmp(const struct nf_conntrack *ct1,
  * 2) You can copy only a part of the source object to the target
  *
  * The current supported flags are:
- * 	- NFCT_CP_ALL that copies the object entirely.
- * 	- NFCT_CP_ORIG and NFCT_CP_REPL that can be used to copy the
+ * 	NFCT_CP_ALL: that copies the object entirely.
+ * 	NFCT_CP_ORIG and NFCT_CP_REPL: that can be used to copy the
  * 	information that identifies a flow in the original and the reply
  * 	direction. This information is usually composed of: source and
  * 	destination IP address; source and destination ports; layer 3
  * 	and 4 protocol number.
- * 	- NFCT_CP_META that copies the metainformation 
- * 	  (all the attributes >= ATTR_TCP_STATE)
+ * 	NFCT_CP_META: that copies the metainformation 
+ * 	(all the attributes >= ATTR_TCP_STATE)
  */
 void nfct_copy(struct nf_conntrack *ct1,
 	       const struct nf_conntrack *ct2,
@@ -771,6 +891,7 @@ void nfct_copy(struct nf_conntrack *ct1,
 	if (flags == NFCT_CP_ALL) {
 		for (i=0; i<ATTR_MAX; i++) {
 			if (test_bit(i, ct2->set)) {
+				assert(copy_attr_array[i]);
 				copy_attr_array[i](ct1, ct2);
 				set_bit(i, ct1->set);
 			}
@@ -796,6 +917,7 @@ void nfct_copy(struct nf_conntrack *ct1,
 	if (flags & NFCT_CP_ORIG) {
 		for (i=0; i<__CP_ORIG_MAX; i++) {
 			if (test_bit(cp_orig_mask[i], ct2->set)) {
+				assert(copy_attr_array[i]);
 				copy_attr_array[cp_orig_mask[i]](ct1, ct2);
 				set_bit(cp_orig_mask[i], ct1->set);
 			}
@@ -817,6 +939,7 @@ void nfct_copy(struct nf_conntrack *ct1,
 	if (flags & NFCT_CP_REPL) {
 		for (i=0; i<__CP_REPL_MAX; i++) {
 			if (test_bit(cp_repl_mask[i], ct2->set)) {
+				assert(copy_attr_array[i]);
 				copy_attr_array[cp_repl_mask[i]](ct1, ct2);
 				set_bit(cp_repl_mask[i], ct1->set);
 			}
@@ -826,6 +949,7 @@ void nfct_copy(struct nf_conntrack *ct1,
 	if (flags & NFCT_CP_META) {
 		for (i=ATTR_TCP_STATE; i<ATTR_MAX; i++) {
 			if (test_bit(i, ct2->set)) {
+				assert(copy_attr_array[i]),
 				copy_attr_array[i](ct1, ct2);
 				set_bit(i, ct1->set);
 			}
@@ -846,6 +970,7 @@ void nfct_copy_attr(struct nf_conntrack *ct1,
 		    const enum nf_conntrack_attr type)
 {
 	if (test_bit(type, ct2->set)) {
+		assert(copy_attr_array[type]);
 		copy_attr_array[type](ct1, ct2);
 		set_bit(type, ct1->set);
 	}
@@ -883,8 +1008,8 @@ void nfct_filter_destroy(struct nfct_filter *filter)
  * @type: filter attribute type
  * @value: pointer to the value of the filter attribute
  *
- * Limitations: You can add up to 256 IPv4 addresses and masks for 
- * NFCT_FILTER_SRC_IPV4 and, similarly, 256 for NFCT_FILTER_DST_IPV4.
+ * Limitations: You can add up to 127 IPv4 addresses and masks for 
+ * NFCT_FILTER_SRC_IPV4 and, similarly, 127 for NFCT_FILTER_DST_IPV4.
  */
 void nfct_filter_add_attr(struct nfct_filter *filter,
 			  const enum nfct_filter_attr type, 
@@ -893,7 +1018,7 @@ void nfct_filter_add_attr(struct nfct_filter *filter,
 	assert(filter != NULL);
 	assert(value != NULL);
 
-	if (type >= NFCT_FILTER_MAX)
+	if (unlikely(type >= NFCT_FILTER_MAX))
 		return;
 
 	if (filter_attr_array[type]) {
@@ -907,6 +1032,8 @@ void nfct_filter_add_attr(struct nfct_filter *filter,
  * @filter: filter object that we want to modify
  * @type: filter attribute type
  * @value: value of the filter attribute using unsigned int (32 bits).
+ *
+ * Limitations: You can add up to 255 protocols which is a reasonable limit.
  */
 void nfct_filter_add_attr_u32(struct nfct_filter *filter,
 			      const enum nfct_filter_attr type,
@@ -934,7 +1061,7 @@ int nfct_filter_set_logic(struct nfct_filter *filter,
 			  const enum nfct_filter_attr type,
 			  const enum nfct_filter_logic logic)
 {
-	if (type >= NFCT_FILTER_MAX) {
+	if (unlikely(type >= NFCT_FILTER_MAX)) {
 		errno = ENOTSUP;
                 return -1;
 	}
